""" Utils module to run OS commands
"""
import os
from kservice.colors import bcolors
from prettytable import PrettyTable


def run_command(operation=None, args=None, service_list=None):
    """ run a specific command based on its operation (start/stop/etc..) """

    if operation == "start":
        required_services = ""

        # check if required service list contains , as separator
        if ',' in args['start'][0]:
            required_services = str(args['start'][0]).split(',')
        else:
            required_services = args['start']

        for service in service_list:
            if (args['namespace'] is False):
                if service.name in required_services:
                    command = f"nohup minikube service {service.name} --url -n {service.namespace} > /tmp/.kservice.{service.namespace}.{service.name}.out 2>&1 &"
                    os.system(command)
            else:
                if (service.name in required_services) and (args['namespace'] == service.namespace):
                    command = f"nohup minikube service {service.name} --url -n {service.namespace} > /tmp/.kservice.{service.namespace}.{service.name}.out 2>&1 &"
                    os.system(command)


    if operation == "stop":
        required_services = ""

        if ',' in args['stop'][0]:
            required_services = str(args['stop'][0]).split(',')
        else:
            required_services = args['stop']

        for service in service_list:
            kill_command = f"pgrep -f \"minikube service {service.name} --url -n {service.namespace}\"|xargs kill -9"
            del_file_command = f"rm -f /tmp/.kservice.{service.namespace}.{service.name}.out"
            if (args['namespace'] is False):
                if service.name in required_services:
                    os.system(kill_command)
                    os.system(del_file_command)
            else:
                if (service.name in required_services) and (args['namespace'] == service.namespace):
                    os.system(kill_command)
                    os.system(del_file_command)


    if operation == "startall":
        for service in service_list:
            command = f"nohup minikube service {service.name} --url -n {service.namespace} > /tmp/.kservice.{service.namespace}.{service.name}.out 2>&1 &"
            
            # check if a specific namespace has been provided
            if (args['namespace'] is not False):
                if (args['namespace'] == service.namespace):
                    os.system(command)
            else:
                os.system(command)
                

    if operation == "stopall":
        for service in service_list:
            kill_command = f"pgrep -f \"minikube service {service.name} --url -n {service.namespace}\"|xargs kill -9"
            del_file_command = f"rm -f /tmp/.kservice.{service.namespace}.{service.name}.out"

            if (args['namespace'] is not False):
                if (args['namespace'] == service.namespace):
                    os.system(kill_command)
                    os.system(del_file_command)
                    continue
            else:
                # if no namespace provided, stop all services in mikube
                os.system(kill_command)
                os.system(del_file_command)


    if operation == "status":
        show_status(service_list)
    

def print_colored(msg=None):
    print(f"{bcolors.HEADER}{msg}{bcolors.ENDC}")

def print_error(msg=None):
    print(f"{bcolors.FAIL}{msg}{bcolors.ENDC}")




def show_status(service_list=None):
    
    t = PrettyTable(['Namespace', 'Service Name', 'URL'])

    for service in service_list:
        service_name = service.name
        namespace = service.namespace

        tmp_file = f"/tmp/.kservice.{namespace}.{service_name}.out"

        if os.path.exists(tmp_file):
            with open(tmp_file, "r") as a_file:
                for line in a_file:
                    stripped_line = line.strip()
                    if "http" in stripped_line:
                        t.add_row([namespace, service_name, stripped_line])
            a_file.close()
        else:
            t.add_row([namespace, service_name, ""])

    print(t)



""" main """
import sys
from kservice.k8s_utils import MK
from kservice.arguments import parse_arguments, return_operation, check_arguments
from kservice.os_utils import print_colored, print_error


def main():
    """ main """
    
    minikube = MK()

    if minikube.check_cluster() is False:
        print_error("Minikube is not running. Exit.")
        exit(0)
    
    arguments = parse_arguments()
    args = vars(arguments.parse_args())

    if check_arguments(args) is False:
        print_error("Please, select only one operation.")
        exit(1)

    if len(sys.argv) == 1:
        arguments.print_help()

    operation = return_operation(args)
    minikube.run(operation, args)


if __name__ == "__main__":
    main()


""" Module used to parse and check arguments
"""
import argparse


def parse_arguments():
    parser = argparse.ArgumentParser(prog='PROG')
    
    group1=parser.add_argument_group("Start/Stop")
    group2=parser.add_argument_group("Startall/Stopall")

    group1.add_argument("--start", help='Start one or more services separated by \',\' or space', nargs='+', default=False)
    group1.add_argument('--stop', help='Stop one or more services separated by \',\' or space', nargs='+', default=False)

    group2.add_argument('--startall', help='Start all services', action='store_true', default=False)
    group2.add_argument('--stopall', help='Stop all services', action='store_true', default=False)
    
    
    parser.add_argument('--status', help='Show current service status', action='store_true', default=False)
    parser.add_argument('-n','--namespace', help='Specificy --namespace', default=False)

    return parser


def check_arguments(args):
    """ Run over the arguments namespace
        If more than 1 argument is True, return False
        We expect only one argument like stop or start, etc. 
    """
    count=0
    for arg in args:
        # don't take into consideration argument namespace
        if arg == 'namespace':
            continue

        if args[arg] is not False:
            count+=1
    
    if count > 1:
        return False

    return True


def return_operation(args):
    """ Since only one argument is expected and previously validated
        This function will return the first argument that is True
        It should be either start, stop, status, startall or stopall
    """
    for arg in args:
        if args[arg] is not False:
            return arg
    


"""A setuptools based setup module.
"""


from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

requirements = (here / "requirements.txt").read_text(encoding="utf-8")



setup(

    name="minikservices",
    version="0.0.6",
    license='MIT',
    description="App to start, stop and view minikube services easily.",
    long_description = long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dszortyka/minikservices",  
    author="Daniel Szortyka",  
    author_email="daniel.szortyka@gmail.com",  
    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        #"Programming Language :: Python :: 3.7",
        #"Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],

    keywords="minikube, k8s, minikube services",
    package_dir={"": "src"},  
    packages=find_packages(where="src"),
    install_requires=[
        'setuptools',
        'wheel',
        'kubernetes==24.2.0',
        'prettytable==3.4.1'
    ],
    python_requires=">=3.9, <4",
    
    entry_points={  
        "console_scripts": [
            "kservice=kservice.main:main",
        ],
    },
    
    project_urls={  
        "Bug Reports": "https://github.com/dszortyka/minikservices/issues",
        "Funding": "https://donate.pypi.org",
        #"Say Thanks!": "http://saythanks.io/to/minikservices",
        "Source": "https://github.com/dszortyka/minikservices/",
    },
)


from kubernetes import client, config
from kservice import os_utils



class Service:
    def __init__(self, namespace=None, name=None):
        self.namespace = namespace
        self.name = name


class MK:
    def __init__(self, host=None):
        self.host = None
        self.all_services = []

    def check_cluster(self, host=None):
        try:
            config.load_kube_config()
            return True
        except:
            return False

    def get_services(self, host=None):
        v1 = client.CoreV1Api()
        ret = v1.list_service_for_all_namespaces()
        for i in ret.items:
            if (i.metadata.namespace == "kube-system") or (i.metadata.name == "kubernetes"):    # by default, don't include kubernetes pod or kube-system namespace in the list of services
                continue

            service = Service(namespace=i.metadata.namespace, name=i.metadata.name)
            self.all_services.append(service)

        return self.all_services

    
    def run(self, operation=None, args=None):
        service_list = self.get_services()
        os_utils.run_command(operation, args, service_list)

        